@@ -1,11 +1,17 @@
+from __future__ import annotations
+
 import os
 from datetime import datetime
+from typing import List
 
 import databases
 from starlette.applications import Starlette
-from starlette.endpoints import HTTPEndpoint
+from starlette.endpoints import HTTPEndpoint, WebSocketEndpoint
 from starlette.responses import JSONResponse, Response
-from starlette.routing import Route, Mount
+from starlette.routing import Route, Mount, WebSocketRoute
+from starlette.types import Scope, Receive, Send
+from starlette.websockets import WebSocket
+
 from dsnetserver import __version__
 from dsnetserver.models import pigeonhole_message_table, broadcast_query_table
 from sqlalchemy import insert
@@ -14,15 +20,35 @@ DATABASE_URL = os.getenv('DS_DATABASE_URL', 'sqlite:///dsnet.db')
 database = databases.Database(DATABASE_URL)
 
 
+class BulletinBoard(WebSocketEndpoint):
+    encoding = 'bytes'
+    connections: List[WebSocket] = list()
+
+    def __init__(self, scope: Scope, receive: Receive, send: Send):
+        super().__init__(scope, receive, send)
+
+    async def on_connect(self, websocket):
+        BulletinBoard.connections.append(websocket)
+        await websocket.accept()
+
+    async def on_disconnect(self, websocket, close_code):
+        BulletinBoard.connections.remove(websocket)
+
+    @classmethod
+    async def broadcast(cls, data):
+        for connection in cls.connections:
+            await connection.send_bytes(data)
+
+
 async def homepage(_):
     return JSONResponse({"message": "Datashare Network Server version %s" % __version__})
 
 
 async def broadcast(request):
     message = await request.body()
-    stmt = insert(broadcast_query_table). \
-        values(received_at=datetime.now(), message=message)
+    stmt = insert(broadcast_query_table).values(received_at=datetime.now(), message=message)
     await database.execute(stmt)
+    await BulletinBoard.broadcast(message)
     return Response()
 
 
@@ -46,6 +72,7 @@ class PigeonHole(HTTPEndpoint):
 routes = [
     Route('/', homepage),
     Route('/bb/broadcast', broadcast, methods=['POST']),
+    WebSocketRoute('/notifications', BulletinBoard),
     Mount('/ph', routes=[
         Route('/{address:str}', PigeonHole, methods=['GET', 'POST']),
     ])
